from datetime import datetime
from math import floor, log10
from opa_tools.config import get_request_dir
from typing import List, Union
import yaml

from .retrieval import get_request
from .transformation import product, unflatten, wrap
from .utils import (
    get_request_location,
    normalize_request_identifier,
    _normalize_request_month,
    ALLOWED_KINDS,
)


REQUEST_DIR = get_request_dir()


def _write_requests(requests, grequest_dir):
    kinds = ALLOWED_KINDS
    with open(grequest_dir / "date.txt", "w") as wf:
        wf.write(datetime.now().isoformat())
    n_digits = floor(log10(len(requests))) + 1
    for i, request in enumerate(requests, start=1):
        request_dir = REQUEST_DIR / grequest_dir / f"{i:0{n_digits}d}"
        request_dir.mkdir()
        for kind, request_kind in zip(kinds, unflatten(request, kinds)):
            with open(request_dir / f"{kind}.yml", "w") as wf:
                yaml.dump(request_kind, wf)


def create_template(
    src: Union[str, List[str]],
    dst: str,
) -> None:
    name, _, month, _ = normalize_request_identifier(dst, None, None)
    new_dir = REQUEST_DIR / month / name
    if new_dir.exists():
        raise IsADirectoryError(
            f"Directory '{new_dir}' exists. Choose a different dst."
        )
    if isinstance(src, str):
        src = [src]
    generator_request = wrap([ # pyright: ignore
        get_request(element, flat=False, single_as_list=False) for element in src
    ])
    new_dir.mkdir()
    new_path = new_dir / "grequest.yaml"
    with open(new_path, "w") as wf:
        yaml.dump(generator_request, wf)
    print(f"File {new_path} created.")


def generate_requests(
    identifier: Union[str, int],
) -> None:
    name, _, month, _ = normalize_request_identifier(identifier, None, None)
    grequest_path = (
        get_request_location(name, month=month, no_exist_ok=True)
        / "grequest.yaml"
    )
    if not grequest_path.exists():
        raise FileNotFoundError(
            f"Generating request attempted to be found in '{grequest_path}', but none found. "
            "Have you run `create_template`?"
        )
    with open(grequest_path, "r") as rf:
        generating_request = yaml.safe_load(rf)
    requests = product(generating_request)
    _write_requests(requests, grequest_path.parent)

